@@ -343,7 +343,7 @@ g_drive_can_poll_for_media (GDrive *drive)
  * @drive: a #GDrive.
  * @flags: flags affecting the unmount if required for eject
  * @cancellable: optional #GCancellable object, %NULL to ignore.
- * @callback: a #GAsyncReadyCallback.
+ * @callback: a #GAsyncReadyCallback, or %NULL.
  * @user_data: a #gpointer.
  * 
  * Ejects a drive.
@@ -411,7 +411,7 @@ g_drive_eject_finish (GDrive        *drive,
  * g_drive_poll_for_media:
  * @drive: a #GDrive.
  * @cancellable: optional #GCancellable object, %NULL to ignore.
- * @callback: a #GAsyncReadyCallback.
+ * @callback: a #GAsyncReadyCallback, or %NULL.
  * @user_data: a #gpointer.
  * 
  * Polls @drive to see if media has been inserted or removed.
@@ -474,5 +474,59 @@ g_drive_poll_for_media_finish (GDrive        *drive,
   return (* iface->poll_for_media_finish) (drive, result, error);
 }
 
+/**
+ * g_drive_get_identifier:
+ * @drive: a #GDrive
+ * @kind: the kind of identifier to return
+ *
+ * Gets the identifier of the given kind for @drive.
+ *
+ * Returns: a newly allocated string containing the
+ *   requested identfier, or %NULL if the #GDrive
+ *   doesn't have this kind of identifier
+ */
+char *
+g_drive_get_identifier (GDrive     *drive,
+			const char *kind)
+{
+  GDriveIface *iface;
+
+  g_return_val_if_fail (G_IS_DRIVE (drive), NULL);
+  g_return_val_if_fail (kind != NULL, NULL);
+
+  iface = G_DRIVE_GET_IFACE (drive);
+
+  if (iface->get_identifier == NULL)
+    return NULL;
+  
+  return (* iface->get_identifier) (drive, kind);
+}
+
+/**
+ * g_drive_enumerate_identifiers:
+ * @drive: a #GDrive
+ *
+ * Gets the kinds of identifiers that @drive has. 
+ * Use g_drive_get_identifer() to obtain the identifiers
+ * themselves.
+ *
+ * Returns: a %NULL-terminated array of strings containing
+ *   kinds of identifiers. Use g_strfreev() to free.
+ */
+char **
+g_drive_enumerate_identifiers (GDrive *drive)
+{
+  GDriveIface *iface;
+
+  g_return_val_if_fail (G_IS_DRIVE (drive), NULL);
+  iface = G_DRIVE_GET_IFACE (drive);
+
+  if (iface->enumerate_identifiers == NULL)
+    return NULL;
+  
+  return (* iface->enumerate_identifiers) (drive);
+}
+
+
 #define __G_DRIVE_C__
 #include "gioaliasdef.c"
