@@ -214,6 +214,7 @@ canonicalize_filename (const char *filename)
 {
   char *canon, *start, *p, *q;
   char *cwd;
+  int i;
   
   if (!g_path_is_absolute (filename))
     {
@@ -226,6 +227,24 @@ canonicalize_filename (const char *filename)
 
   start = (char *)g_path_skip_root (canon);
 
+  /* POSIX allows double slashes at the start to
+   * mean something special (as does windows too).
+   * So, "//" != "/", but more than two slashes
+   * is treated as "/".
+   */
+  i = 0;
+  for (p = start - 1;
+       (p >= canon) &&
+	 G_IS_DIR_SEPARATOR (*p);
+       p--)
+    i++;
+  if (i > 2)
+    {
+      i -= 1;
+      start -= i;
+      memmove (start, start+i, strlen (start+i)+1);
+    }
+  
   p = start;
   while (*p != 0)
     {
@@ -348,7 +367,7 @@ name_is_valid_for_display (const char *string,
 			   gboolean    is_valid_utf8)
 {
   char c;
-  
+
   if (!is_valid_utf8 &&
       !g_utf8_validate (string, -1, NULL))
     return FALSE;
@@ -372,7 +391,8 @@ g_local_file_get_parse_name (GFile *file)
   char *roundtripped_filename;
   gboolean free_utf8_filename;
   gboolean is_valid_utf8;
-
+  char *escaped_path;
+  
   filename = G_LOCAL_FILE (file)->filename;
   if (get_filename_charset (&charset))
     {
@@ -402,7 +422,6 @@ g_local_file_get_parse_name (GFile *file)
 	}
     }
 
-
   if (utf8_filename != NULL &&
       name_is_valid_for_display (utf8_filename, is_valid_utf8))
     {
@@ -413,7 +432,16 @@ g_local_file_get_parse_name (GFile *file)
     }
   else
     {
-      parse_name = g_filename_to_uri (filename, NULL, NULL);
+      escaped_path = g_uri_escape_string (filename,
+					  G_URI_RESERVED_CHARS_ALLOWED_IN_PATH_ELEMENT "/",
+					  TRUE);
+      parse_name = g_strconcat ("file://",
+				(*escaped_path != '/') ? "/" : "",
+				escaped_path,
+				NULL);
+      
+      g_free (escaped_path);
+
       if (free_utf8_filename)
 	g_free (utf8_filename);
     }
@@ -1598,7 +1626,7 @@ g_local_file_trash (GFile         *file,
       
       g_set_error (error, G_IO_ERROR,
 		   g_io_error_from_errno (errno),
-		   _("Unable to create trashed file: %s"),
+		   _("Unable to create trashing info file: %s"),
 		   g_strerror (errno));
       return FALSE;
     }
@@ -1808,7 +1836,7 @@ g_local_file_move (GFile                  *source,
 	  g_set_error (error,
 		       G_IO_ERROR,
 		       G_IO_ERROR_EXISTS,
-		       _("Target file already exists"));
+		       _("Target file exists"));
 	  return FALSE;
 	}
     }
