@@ -2,7 +2,7 @@
  *
  * gconvert.c: Convert between character sets using iconv
  * Copyright Red Hat Inc., 2000
- * Authors: Havoc Pennington <hp@redhat.com>, Owen Taylor <otaylor@redhat.com
+ * Authors: Havoc Pennington <hp@redhat.com>, Owen Taylor <otaylor@redhat.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -22,15 +22,19 @@
 
 #include "config.h"
 
+#include "glib.h"
+
+#ifndef G_OS_WIN32
 #include <iconv.h>
+#endif
 #include <errno.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 
-#include "glib.h"
 #include "gprintfint.h"
 #include "gthreadprivate.h"
+#include "gunicode.h"
 
 #ifdef G_PLATFORM_WIN32
 #define STRICT
@@ -49,6 +53,10 @@
 
 #include "galias.h"
 
+#ifdef G_OS_WIN32
+#include "win_iconv.c"
+#endif
+
 GQuark 
 g_convert_error_quark (void)
 {
@@ -88,7 +96,8 @@ try_to_aliases (const char **to_aliases,
   return FALSE;
 }
 
-extern const char ** G_GNUC_INTERNAL _g_charset_get_aliases (const char *canonical_name);
+G_GNUC_INTERNAL extern const char ** 
+_g_charset_get_aliases (const char *canonical_name);
 
 /**
  * g_iconv_open:
@@ -156,7 +165,7 @@ g_iconv_open (const gchar  *to_codeset,
  * 
  * Return value: count of non-reversible conversions, or -1 on error
  **/
-size_t 
+gsize 
 g_iconv (GIConv   converter,
 	 gchar  **inbuf,
 	 gsize   *inbytes_left,
@@ -226,7 +235,7 @@ iconv_cache_init (void)
 }
 
 
-/**
+/*
  * iconv_cache_bucket_new:
  * @key: cache key
  * @cd: iconv descriptor
@@ -234,15 +243,17 @@ iconv_cache_init (void)
  * Creates a new cache bucket, inserts it into the cache and
  * increments the cache size.
  *
+ * This assumes ownership of @key.
+ *
  * Returns a pointer to the newly allocated cache bucket.
  **/
 static struct _iconv_cache_bucket *
-iconv_cache_bucket_new (const gchar *key, GIConv cd)
+iconv_cache_bucket_new (gchar *key, GIConv cd)
 {
   struct _iconv_cache_bucket *bucket;
   
   bucket = g_new (struct _iconv_cache_bucket, 1);
-  bucket->key = g_strdup (key);
+  bucket->key = key;
   bucket->refcount = 1;
   bucket->used = TRUE;
   bucket->cd = cd;
@@ -259,7 +270,7 @@ iconv_cache_bucket_new (const gchar *key, GIConv cd)
 }
 
 
-/**
+/*
  * iconv_cache_bucket_expire:
  * @node: cache bucket's node
  * @bucket: cache bucket
@@ -304,7 +315,7 @@ iconv_cache_bucket_expire (GList *node, struct _iconv_cache_bucket *bucket)
 }
 
 
-/**
+/*
  * iconv_cache_expire_unused:
  *
  * Expires as many unused cache buckets as it needs to in order to get
@@ -335,13 +346,24 @@ open_converter (const gchar *to_codeset,
 		GError     **error)
 {
   struct _iconv_cache_bucket *bucket;
-  gchar *key;
+  gchar *key, *dyn_key, auto_key[80];
   GIConv cd;
+  gsize len_from_codeset, len_to_codeset;
   
   /* create our key */
-  key = g_alloca (strlen (from_codeset) + strlen (to_codeset) + 2);
-  _g_sprintf (key, "%s:%s", from_codeset, to_codeset);
-  
+  len_from_codeset = strlen (from_codeset);
+  len_to_codeset = strlen (to_codeset);
+  if (len_from_codeset + len_to_codeset + 2 < sizeof (auto_key))
+    {
+      key = auto_key;
+      dyn_key = NULL;
+    }
+  else
+    key = dyn_key = g_malloc (len_from_codeset + len_to_codeset + 2);
+  memcpy (key, from_codeset, len_from_codeset);
+  key[len_from_codeset] = ':';
+  strcpy (key + len_from_codeset + 1, to_codeset);
+
   G_LOCK (iconv_cache_lock);
   
   /* make sure the cache has been initialized */
@@ -350,6 +372,8 @@ open_converter (const gchar *to_codeset,
   bucket = g_hash_table_lookup (iconv_cache, key);
   if (bucket)
     {
+      g_free (dyn_key);
+
       if (bucket->used)
         {
           cd = g_iconv_open (to_codeset, from_codeset);
@@ -378,12 +402,15 @@ open_converter (const gchar *to_codeset,
   else
     {
       cd = g_iconv_open (to_codeset, from_codeset);
-      if (cd == (GIConv) -1)
-        goto error;
+      if (cd == (GIConv) -1) 
+	{
+	  g_free (dyn_key);
+	  goto error;
+	}
       
       iconv_cache_expire_unused ();
-      
-      bucket = iconv_cache_bucket_new (key, cd);
+
+      bucket = iconv_cache_bucket_new (dyn_key ? dyn_key : g_strdup (key), cd);
     }
   
   g_hash_table_insert (iconv_open_hash, cd, bucket->key);
@@ -553,13 +580,13 @@ g_convert_with_iconv (const gchar *str,
   gchar *dest;
   gchar *outp;
   const gchar *p;
-  const gchar *shift_p = NULL;
   gsize inbytes_remaining;
   gsize outbytes_remaining;
   gsize err;
   gsize outbuf_size;
   gboolean have_error = FALSE;
   gboolean done = FALSE;
+  gboolean reset = FALSE;
   
   g_return_val_if_fail (converter != (GIConv) -1, NULL);
      
@@ -575,9 +602,12 @@ g_convert_with_iconv (const gchar *str,
 
   while (!done && !have_error)
     {
-      err = g_iconv (converter, (char **)&p, &inbytes_remaining, &outp, &outbytes_remaining);
+      if (reset)
+        err = g_iconv (converter, NULL, &inbytes_remaining, &outp, &outbytes_remaining);
+      else
+        err = g_iconv (converter, (char **)&p, &inbytes_remaining, &outp, &outbytes_remaining);
 
-      if (err == (size_t) -1)
+      if (err == (gsize) -1)
 	{
 	  switch (errno)
 	    {
@@ -587,7 +617,7 @@ g_convert_with_iconv (const gchar *str,
 	      break;
 	    case E2BIG:
 	      {
-		size_t used = outp - dest;
+		gsize used = outp - dest;
 		
 		outbuf_size *= 2;
 		dest = g_realloc (dest, outbuf_size);
@@ -613,11 +643,10 @@ g_convert_with_iconv (const gchar *str,
 	}
       else 
 	{
-	  if (!shift_p)
+	  if (!reset)
 	    {
 	      /* call g_iconv with NULL inbuf to cleanup shift state */
-	      shift_p = p;
-	      p = NULL;
+	      reset = TRUE;
 	      inbytes_remaining = 0;
 	    }
 	  else
@@ -625,9 +654,6 @@ g_convert_with_iconv (const gchar *str,
 	}
     }
 
-  if (shift_p)
-    p = shift_p;
-
   *outp = '\0';
   
   if (bytes_read)
@@ -861,11 +887,11 @@ g_convert_with_fallback (const gchar *str,
 
   while (!done && !have_error)
     {
-      size_t inbytes_tmp = inbytes_remaining;
+      gsize inbytes_tmp = inbytes_remaining;
       err = g_iconv (cd, (char **)&p, &inbytes_tmp, &outp, &outbytes_remaining);
       inbytes_remaining = inbytes_tmp;
 
-      if (err == (size_t) -1)
+      if (err == (gsize) -1)
 	{
 	  switch (errno)
 	    {
@@ -874,7 +900,7 @@ g_convert_with_fallback (const gchar *str,
 	      break;
 	    case E2BIG:
 	      {
-		size_t used = outp - dest;
+		gsize used = outp - dest;
 
 		outbuf_size *= 2;
 		dest = g_realloc (dest, outbuf_size);
@@ -1031,7 +1057,8 @@ strdup_len (const gchar *string,
  * 
  * Converts a string which is in the encoding used for strings by
  * the C runtime (usually the same as that used by the operating
- * system) in the current locale into a UTF-8 string.
+ * system) in the <link linkend="setlocale">current locale</link> into a
+ * UTF-8 string.
  * 
  * Return value: The converted string, or %NULL on an error.
  **/
@@ -1071,7 +1098,7 @@ g_locale_to_utf8 (const gchar  *opsysstring,
  * 
  * Converts a string from UTF-8 to the encoding used for strings by
  * the C runtime (usually the same as that used by the operating
- * system) in the current locale.
+ * system) in the <link linkend="setlocale">current locale</link>.
  * 
  * Return value: The converted string, or %NULL on an error.
  **/
@@ -1126,18 +1153,19 @@ filename_charset_cache_free (gpointer data)
  * have no effect.
  *
  * <envar>G_FILENAME_ENCODING</envar> may be set to a comma-separated list 
- * of character set names. The special token "@locale" is taken to mean the 
- * character set for the current locale. If <envar>G_FILENAME_ENCODING</envar> 
- * is not set, but <envar>G_BROKEN_FILENAMES</envar> is, the character set of 
- * the current locale is taken as the filename encoding. If neither environment
- * variable is set, UTF-8 is taken as the filename encoding, but the character
+ * of character set names. The special token "&commat;locale" is taken to 
+ * mean the character set for the <link linkend="setlocale">current 
+ * locale</link>. If <envar>G_FILENAME_ENCODING</envar> is not set, but 
+ * <envar>G_BROKEN_FILENAMES</envar> is, the character set of the current 
+ * locale is taken as the filename encoding. If neither environment variable 
+ * is set, UTF-8 is taken as the filename encoding, but the character
  * set of the current locale is also put in the list of encodings.
  *
  * The returned @charsets belong to GLib and must not be freed.
  *
  * Note that on Unix, regardless of the locale character set or
- * <envar>G_FILENAME_ENCODING</envar> value, the actual file names present on a
- * system might be in any random encoding or just gibberish.
+ * <envar>G_FILENAME_ENCODING</envar> value, the actual file names present 
+ * on a system might be in any random encoding or just gibberish.
  *
  * Return value: %TRUE if the filename encoding is UTF-8.
  * 
@@ -1281,7 +1309,8 @@ _g_convert_thread_init (void)
  * 
  * Converts a string which is in the encoding used by GLib for
  * filenames into a UTF-8 string. Note that on Windows GLib uses UTF-8
- * for filenames.
+ * for filenames; on other platforms, this function indirectly depends on 
+ * the <link linkend="setlocale">current locale</link>.
  * 
  * Return value: The converted string, or %NULL on an error.
  **/
@@ -1344,7 +1373,9 @@ g_filename_to_utf8 (const gchar *opsysstring,
  *                 errors. Any of the errors in #GConvertError may occur.
  * 
  * Converts a string from UTF-8 to the encoding GLib uses for
- * filenames. Note that on Windows GLib uses UTF-8 for filenames.
+ * filenames. Note that on Windows GLib uses UTF-8 for filenames;
+ * on other platforms, this function indirectly depends on the 
+ * <link linkend="setlocale">current locale</link>.
  * 
  * Return value: The converted string, or %NULL on an error.
  **/
@@ -1972,44 +2003,6 @@ g_uri_list_extract_uris (const gchar *uri_list)
   return result;
 }
 
-static gchar *
-make_valid_utf8 (const gchar *name)
-{
-  GString *string;
-  const gchar *remainder, *invalid;
-  gint remaining_bytes, valid_bytes;
-  
-  string = NULL;
-  remainder = name;
-  remaining_bytes = strlen (name);
-  
-  while (remaining_bytes != 0) 
-    {
-      if (g_utf8_validate (remainder, remaining_bytes, &invalid)) 
-	break;
-      valid_bytes = invalid - remainder;
-    
-      if (string == NULL) 
-	string = g_string_sized_new (remaining_bytes);
-
-      g_string_append_len (string, remainder, valid_bytes);
-      /* append U+FFFD REPLACEMENT CHARACTER */
-      g_string_append (string, "\357\277\275");
-      
-      remaining_bytes -= valid_bytes + 1;
-      remainder = invalid + 1;
-    }
-  
-  if (string == NULL)
-    return g_strdup (name);
-  
-  g_string_append (string, remainder);
-
-  g_assert (g_utf8_validate (string->str, -1, NULL));
-  
-  return g_string_free (string, FALSE);
-}
-
 /**
  * g_filename_display_basename:
  * @filename: an absolute pathname in the GLib file name encoding
@@ -2110,7 +2103,7 @@ g_filename_display_name (const gchar *filename)
    * by a question mark
    */
   if (!display_name) 
-    display_name = make_valid_utf8 (filename);
+    display_name = _g_utf8_make_valid (filename);
 
   return display_name;
 }
